socket = require('socket')
require('myrdb')

local timer
local server
local path = '/opt/ntcagent/fmc.bin'
local TYPE = 'application/octet-stream'
local CHUNK = 4096
local devices = {}


local function getChildDeviceID(serial)
   if devices[serial] then return devices[serial] end
   http:clear()
   if http:post('300,' .. serial) <= 0 then return nil end
   local resp = http:response()
   if string.sub(resp, 1, 2) == "50" then
      http:clear()
      if http:post('346,' .. 'FMC (' .. serial .. '),,' .. serial) <= 0 then
         return nil
      end
      resp = http:response()
      if string.sub(resp, 1, 3) == '884' then
         local ID = string.match(resp, '%d+,%d+,(%d+)')
         local msg = '302,'..ID..','..serial..'\n342,'..c8y.ID..','..ID
         if http:post(msg) < 0 then return nil end
         devices[serial] = ID
         return ID
      end
   elseif string.sub(resp, 1, 3) == "800" then
      return string.match(resp, "%d+,%d+,(%d+)")
   end
   return nil
end


local function myevent(msgid, tbl)
   return msgid .. ',' .. utcTime() .. ',' .. table.concat(tbl, ',')
end


local function bailout(file, client, errmsg, logmsg, filename, size)
   if file then file:close() end
   if client then client:close() end
   srError('FMC: ' .. errmsg)
   local now = utcTime()
   local tbl = {c8y.ID, errmsg, filename, size}
   c8y:send('343,' .. now .. ',' .. table.concat(tbl, ','), 1)
   c8y:send('344,' .. now .. ',' .. c8y.ID .. ',' .. logmsg, 1)
end


function poll()
   local file
   local client = server:accept()
   if not client then return end

   local peer = client:getpeername()
   srInfo('FMC: connected ' .. peer)
   c8y:send(myevent('344', {c8y.ID, 'Connected ' .. peer}))
   client:settimeout(60)
   local header, err = client:receive(12)
   if err then
      bailout(file, client, 'Sign-On header: ' .. err,
              'Disconnected ' .. peer, '', -1)
      return
   end
   local tbl = {string.byte(header, 9, 12)}
   local size = tbl[4] * 16777216 + tbl[3] * 65536 + tbl[2] * 256 + tbl[1]
   local payload
   payload, err = client:receive(size)
   if err then
      bailout(file, client, 'Sign-On data: ' .. err,
              'Disconnected ' .. peer, '', -1)
      return
   end
   size, err = client:send(header .. payload)
   local filename = string.sub(payload, 1, -2)
   local serial = string.match(filename, "<([^>]+)>")
   if serial == nil then
      bailout(file, client, 'Match serial failed',
              'Disconnected ' .. peer, filename, -1)
      return
   end
   local ID = getChildDeviceID(serial)
   if ID == nil then
      bailout(file, client, 'Create device failed',
              'Disconnected ' .. peer, filename, -1)
      return
   end
   if err then
      bailout(file, client, 'Ack: ' .. err,
              'Disconnected ' .. peer, filename, -1)
      return
   end
   header, err = client:receive(12)
   if err then
      bailout(file, client, 'Binary header: ' .. err,
              'Disconnected ' .. peer, filename, -1)
      return
   end
   tbl = {string.byte(header, 9, 12)}
   local size = tbl[4] * 16777216 + tbl[3] * 65536 + tbl[2] * 256 + tbl[1] - 1
   file = io.open(path, 'wb')
   if not file then
      bailout(file, client, 'File write error',
              'Disconnected ' .. peer, filename, size)
      return
   end
   srDebug('FMC: ' .. filename .. ' ' .. size)
   local originsize = size
   while size > 0 do
      local chunk = size > CHUNK and CHUNK or size
      payload, err = client:receive(chunk)
      if payload then
         file:write(payload)
         size = size - chunk
      else
         bailout(file, client, 'Binary data: ' .. err,
                 'Disconnected ' .. peer, filename, originsize)
         return
      end
   end
   client:close()
   file:close()
   c8y:send(myevent('344', {c8y.ID, 'Disconnected ' .. peer}), 1)
   local dest = rdbGetStr('uboot.sn') .. ' ' .. filename
   if c8y:postf(dest, TYPE, path) == -1 then
      srError('FMC: upload error')
      c8y:send(myevent('343', {c8y.ID, 'Upload failed',
                               filename, originsize}), 1)
   else
      local l = string.match(c8y.resp, '"self":"(%S+/inventory/binaries/%d+)"')
      c8y:send(myevent('347', {ID, filename, originsize, l}), 1)
   end
end


function addDevice(r)
   devices[r:value(3)] = r:value(2)
end


function init()
   c8y:addMsgHandler(883, 'addDevice')
   timer = c8y:addTimer(1 * 1000, 'poll')
   server = socket.bind('0.0.0.0', 2700)
   server:settimeout(1)
   timer:start()
   return 0
end
