require('myrdb')

local timer
local t = 0
local deviceID
local serial = "Testomat-"..c8y.ID
local logFile = "/tmp/heyl.log"
local fileOffset = 0
local MAX_FILE_SIZE = 100000

--Lua 5.1/5.2 compatibility.
if not table.pack then
    function table.pack (...)
        return {n=select('#',...); ...}
    end
end
if not table.unpack then
   function table.unpack (t, i)
      i = i or 1
      if t[i] ~= nil then
         return t[i], unpack(t, i + 1)
      end
   end
end

local function getChildDeviceID()
   if deviceID then return deviceID end
   local ID
   http:clear()
   if http:post('339,'..serial) <= 0 then return nil end
   local resp = http:response()
   if string.sub(resp, 1, 2) == "50" then
      http:clear()
      if http:post('363,Testomat,1440') <= 0 then return nil end
      resp = http:response()
      if string.sub(resp, 1, 3) == '893' then
         ID = string.match(resp, '%d+,%d+,(%d+)')
         local msg = '341,'..ID..','..serial..'\n342,'..c8y.ID..','..ID
         if http:post(msg) < 0 then return nil end
      end
   elseif string.sub(resp, 1, 3) == "800" then
      ID = string.match(resp, "%d+,%d+,(%d+)")
   end
   if ID then
      deviceID = ID
      srInfo("HEYL Device ID is "..ID)
   else
      srError("HEYL Could not retrieve ID")
   end
   return ID
end

local function getMessages(str)
   local messages = {}
   for msg in str:gmatch("\2([^\2-\3]+)\3") do
      srDebug("HEYL recv: "..msg)
      local t = {}
      for value in msg:gmatch("([^,]+)") do
         table.insert(t, value)
      end
      table.insert(messages,t)
   end
   return messages
end

local function clearLogFile()
   os.execute("killall socat; sleep 1")
   os.execute("rm "..logFile)
   os.execute("socat -u /dev/ttyAPP4,b9600,cs8,parenb=0,cstopb=1,raw,echo=0 "
      ..logFile.." &")
   filePos = 0
end

local function getMeasurementType(s)
   local postfix
   if #s == 1 then
      postfix = s:sub(1,1):upper()
   else
      postfix = s:sub(1,1):upper()..s:sub(2,-1):lower()
   end
   return "c8y_"..postfix
end

local function send(messages)
   for i, m in ipairs(messages) do
      local timestamp = utcTime()
      if m[1] == "ME" then
         local measType = getMeasurementType(m[2])
         local unit = m[8]
         c8y:send(table.concat({
                                 '337',
                                 timestamp,
                                 deviceID,
                                 measType,
                                 string.gsub(m[5]..m[6],'-',''),
                                 string.match(m[7],"(%-?%d+%.?%d*)"),
                                 unit,
                                 measType
                              }, ','))
         if m[9] and m[10] and m[9] ~= '-' and m[10] ~= '-' then
            local series = string.match(m[9],"(.*):$")
            local limit = string.match(m[10],"(%-?%d+%.?%d*)")
            c8y:send(table.concat({
                                    '337',
                                    timestamp,
                                    deviceID,
                                    measType,
                                    string.match(m[9],"(.*):$"),
                                    string.match(m[10],"(%-?%d+%.?%d*)"),
                                    unit,
                                    measType
                                 }, ','))
         end
         if m[11] and m[12] and m[11] ~= '-' and m[12] ~= '-' then
            local series = string.match(m[11],"(.*):$")
            local limit = string.match(m[12],"(%-?%d+%.?%d*)")
            c8y:send(table.concat({
                                    '337',
                                    timestamp,
                                    deviceID,
                                    measType,
                                    string.match(m[11],"(.*):$"),
                                    string.match(m[12],"(%-?%d+%.?%d*)"),
                                    unit,
                                    measType
                                 }, ','))
         end
      elseif m[1] == "AL" then
         local alarmType
         local code = string.match(m[2], "(%d+)")
         if code then
            alarmType = "c8y_ErrorCode"..code.."Alarm"
         else
            alarmType = "c8y_TestomatAlarm"
         end
         c8y:send(table.concat({
                                 '334',
                                 timestamp,
                                 deviceID,
                                 alarmType,
                                 "CRITICAL",
                                 m[2]
                              }, ','), 1)
      end
   end
end

local function replaceNullCharsWithSpaces(str)
   local t = table.pack(str:byte(1,#str))
   for i,v in ipairs(t) do
      if v == 0 then t[i] = string.byte(' ') end
   end
   return string.char(table.unpack(t))
end

function checkForData()
   local interval = rdbGetInt("service.cumulocity.heyl.interval")
   local current_time = os.time()
   if interval == 0 or t + interval > current_time then
      return
   end
   t = current_time
   local file = io.open(logFile, "r")
   if not file then
      srError("HEYL Can't open file \""..logFile.."\"")
      return
   end
   file:seek("cur",fileOffset)
   local fs = file:read("*a")
   if fileOffset ~= file:seek() then
      fileOffset = file:seek()
      file:close()
      fs = replaceNullCharsWithSpaces(fs)
      send(getMessages(fs))
      if fileOffset > MAX_FILE_SIZE then
         clearLogFile()
         srDebug("HEYL Log file cleared")
      end
   else
      file:close()
   end
end

function init()
   os.execute("sys -r rs232")
   clearLogFile()
   if getChildDeviceID() then
      timer = c8y:addTimer(1 * 1000, 'checkForData')
      timer:start()
   end
   return 0
end