commProtocolTbl = {
   ["P"] = {
      denote="ASCII, polled",
      responseTerminator="\\r\\n"
   },
   ["S"] = {
      denote="SDI-12 v1.3",
      responseTerminator="!"
   },
}

queryCmdsTbl = {
   ["R1"] = {
      denote = "Wind message query command",
      enabled = true,
      fragments = {
         ["WindSpeed"] = {
            enabled = true,
            series = {
               ["minimum"] = {
                  enabled = true,
                  abbrev = "Sn",
                  units = {
                     ["M"] = "m/s",
                     ["K"] = "km/h",
                     ["S"] = "mph",
                     ["N"] = "knots",
                     ["#"] = "alarm"
                  },
                  alarm = {
                     type = "WindSensorAlarm",
                     text = "Invalid data from Wind Sensor",
                     severity = "MAJOR",
                     status = "ACTIVE"
                  }
               },
               ["average"] = {
                  enabled = true,
                  abbrev = "Sm",
                  units = {
                     ["M"] = "m/s",
                     ["K"] = "km/h",
                     ["S"] = "mph",
                     ["N"] = "knots"
                  }
               },
               ["maximum"] = {
                  enabled = true,
                  abbrev = "Sx",
                  units = {
                     ["M"] = "m/s",
                     ["K"] = "km/h",
                     ["S"] = "mph",
                     ["N"] = "knots"
                  }
               }
            }
         },
         ["WindDirection"] = {
            enabled = true,
            series = {
               ["minimum"] = {
                  enabled = true,
                  abbrev = "Dn",
                  units = {
                     ["D"] = "deg"
                  }
               },
               ["average"] = {
                  enabled = true,
                  abbrev = "Dm",
                  units = {
                     ["D"] = "deg"
                  }
               },
               ["maximum"] = {
                  enabled = true,
                  abbrev = "Dx",
                  units = {
                     ["D"] = "deg"
                  }
               }
            }
         }
      }
   },

   ["R2"] = {
      denote = "Pressure, temperature and humidity message query command",
      enabled = true,
      fragments = {
         ["AirPressure"] = {
            enabled = true,
            series = {
               ["air_pressure"] = {
                  enabled = true,
                  abbrev = "Pa",
                  units = {
                     ["H"] = "hPa",
                     ["P"] = "Pa",
                     ["B"] = "bar",
                     ["M"] = "mmHg",
                     ["I"] = "inHg",
                     ["#"] = "alarm"
                  },
                  alarm = {
                     type = "AirPressureSensorAlarm",
                     text = "Invalid data from Air Pressure Sensor",
                     severity = "MAJOR",
                     status = "ACTIVE"
                  }
               }
            }
         },
         ["Temperature"] = {
            enabled = true,
            series = {
               ["air_temperature"] = {
                  enabled = true,
                  abbrev = "Ta",
                  units = {
                     ["C"] = "C",
                     ["F"] = "F",
                     ["#"] = "alarm"
                  },
                  alarm = {
                     type = "AirTemperatureSensorAlarm",
                     text = "Invalid data from Air Temperature Sensor",
                     severity = "MAJOR",
                     status = "ACTIVE"
                  }
               },
               ["internal_temperature"] = {
                  enabled = true,
                  abbrev = "Tp",
                  units = {
                     ["C"] = "C",
                     ["F"] = "F",
                     ["#"] = "alarm"
                  },
                  alarm = {
                     type = "InternalTemperatureSensorAlarm",
                     text = "Invalid data from Internal Temperature Sensor",
                     severity = "MAJOR",
                     status = "ACTIVE"
                  }
               }
            }
         },
         ["RelativeHumidity"] = {
            enabled = true,
            series = {
               ["relative_humidity"] = {
                  enabled = true,
                  abbrev = "Ua",
                  units = {
                     ["P"] = "%RH",
                     ["#"] = "alarm"
                  },
                  alarm = {
                     type = "RelativeHumiditySensorAlarm",
                     text = "Invalid data from Relative Humidity Sensor",
                     severity = "MAJOR",
                     status = "ACTIVE"
                  }
               }
            }
         }
      }
   },

   ["R3"] = {
      denote = "Precipitation message query command",
      enabled = true,
      fragments = {
         ["Rain"] = {
            enabled = true,
            series = {
               ["rain_accumulation"] = {
                  enabled = true,
                  abbrev = "Rc",
                  units = {
                     ["M"] = "mm",
                     ["I"] = "in",
                     ["#"] = "alarm"
                  },
                  alarm = {
                     type = "PrecipitationSensorAlarm",
                     text = "Invalid data from Precipitation Sensor",
                     severity = "MAJOR",
                     status = "ACTIVE"
                  }
               },
               ["rain_duration"] = {
                  enabled = true,
                  abbrev = "Rd",
                  units = {
                     ["s"] = "s"
                  }
               },
               ["rain_intensity"] = {
                  enabled = true,
                  abbrev = "Ri",
                  units = {
                     ["M"] = "mm/h",
                     ["I"] = "in/h"
                  }
               },
               ["rain_peak_intensity"] = {
                  enabled = true,
                  abbrev = "Rp",
                  units = {
                     ["M"] = "mm/h",
                     ["I"] = "in/h"
                  }
               }
            }
         },
         ["Hail"] = {
            enabled = true,
            series = {
               ["hail_accumulation"] = {
                  enabled = true,
                  abbrev = "Hc",
                  units = {
                     ["M"] = "hits/cm^2",
                     ["I"] = "hits/in^2",
                     ["H"] = "hits"
                  }
               },
               ["hail_duration"] = {
                  enabled = true,
                  abbrev = "Hd",
                  units = {
                     ["s"] = "s"
                  }
               },
               ["hail_intensity"] = {
                  enabled = true,
                  abbrev = "Hc",
                  units = {
                     ["M"] = "hits/cm^2h",
                     ["I"] = "hits/in^2h",
                     ["H"] = "hits/h"
                  }
               },
               ["hail_peak_intensity"] = {
                  enabled = true,
                  abbrev = "Hp",
                  units = {
                     ["M"] = "hits/cm^2h",
                     ["I"] = "hits/in^2h",
                     ["H"] = "hits/h"
                  }
               }
            }
         }
      }
   },

   ["R5"] = {
      denote = "Supervisor message query command",
      enabled = true,
      fragments = {
         ["Supervisor"] = {
            enabled = true,
            series = {
               ["heating_temperature"] = {
                  enabled = true,
                  abbrev = "Th",
                  units = {
                     ["C"] = "C",
                     ["F"] = "F",
                     ["#"] = "alarm"
                  },
                  alarm = {
                     type = "HeatingTemperatureSensorAlarm",
                     text = "Invalid data from Heating Temperature Sensor",
                     severity = "MAJOR",
                     status = "ACTIVE"
                  }
               },
               ["heating_voltage"] = {
                  enabled = true,
                  abbrev = "Vh",
                  units = {
                     ["V"] = "V",
                     ["W"] = "V",
                     ["F"] = "V"
                  }
               },
               ["supply_voltage"] = {
                  enabled = true,
                  abbrev = "Vs",
                  units = {
                     ["V"] = "V"
                  }
               },
               ["3.5V_reference_voltage"] = {
                  enabled = true,
                  abbrev = "Vr",
                  units = {
                     ["V"] = "V"
                  }
               }
            }
         }
      }
   }
}
