require("vaisala/wxt530")
require('myrdb')

local r1Timer, r2Timer, r3Timer, r5Timer
local t1, t2, t3, t5  = 0, 0, 0, 0
local is404Processed = false
local wxt530 = WXT530:new()

function init()
   rdbSet("service.cumulocity.wxt530.c8y_id", 0)
   c8y:addMsgHandler(879, 'getExistent_c8yID')
   c8y:addMsgHandler(50, 'process404ForGetExternalID')
   c8y:addMsgHandler(880, 'getCreated_c8yID')
   wxt530:getExternalID()

   return 0
end

function getExistent_c8yID(r)
   if (r:value(3) == wxt530:getDeviceID()) then
      rdbSet("service.cumulocity.wxt530.c8y_id", r:value(2))
      wxt530:set_c8yID(r:value(2))
      sendMeasurement()
   end
end

function process404ForGetExternalID(r)
   if (r:value(2) == "404" and r:value(3) == "Not Found"
      and is404Processed == false) then
         is404Processed = true
         wxt530:postCreateDevice()
   end
end

function getCreated_c8yID(r)
   if (string.match(r:value(3),"("..wxt530:getSerialNumber()..")") ~= nil) then
      rdbSet("service.cumulocity.wxt530.c8y_id", r:value(2))
      wxt530:set_c8yID(r:value(2))
      wxt530:postCreateExternalID()
      wxt530:postAddChildDevice()
      sendMeasurement()
   end
end

function sendMeasurement()
   if (rdbGetInt("service.cumulocity.wxt530.c8y_id") > 0) then
      r1Timer = c8y:addTimer(15*1000, 'send_r1')
      r2Timer = c8y:addTimer(15*1000, 'send_r2')
      r3Timer = c8y:addTimer(15*1000, 'send_r3')
      r5Timer = c8y:addTimer(15*1000, 'send_r5')
      r1Timer:start()
      r2Timer:start()
      r3Timer:start()
      r5Timer:start()
   end
end

function send_r1()
   local interval = rdbGetInt("service.cumulocity.wxt530.r1.interval")
   local current_time = os.time()
   if interval == 0 or t1 + interval > current_time then
      return
   end
   t1 = current_time
   wxt530:send_r1()
end

function send_r2()
   local interval = rdbGetInt("service.cumulocity.wxt530.r2.interval")
   local current_time = os.time()
   if interval == 0 or t2 + interval > current_time then
      return
   end
   t2 = current_time
   wxt530:send_r2()
end

function send_r3()
   local interval = rdbGetInt("service.cumulocity.wxt530.r3.interval")
   local current_time = os.time()
   if interval == 0 or t3 + interval > current_time then
      return
   end
   t3 = current_time
   wxt530:send_r3()
end

function send_r5()
   local interval = rdbGetInt("service.cumulocity.wxt530.r5.interval")
   local current_time = os.time()
   if interval == 0 or t5 + interval > current_time then
      return
   end
   t5 = current_time
   wxt530:send_r5()
end
